@@ -1505,7 +1505,7 @@ _gegl_buffer_get_unlocked (GeglBuffer          *buffer,
                                    rowstride);
           g_free (sample_buf);
         }
-      else
+      else if (buf_height && buf_width)
         {
           sample_buf = g_malloc (buf_height * buf_width * bpp);
           
@@ -1559,18 +1559,6 @@ gegl_buffer_get (GeglBuffer          *buffer,
   gegl_buffer_unlock (buffer);
 }
 
-void
-gegl_buffer_sample_cleanup (GeglBuffer *buffer)
-{
-  g_return_if_fail (GEGL_IS_BUFFER (buffer));
-
-  if (buffer->sampler)
-    {
-      g_object_unref (buffer->sampler);
-      buffer->sampler = NULL;
-    }
-}
-
 static void
 gegl_buffer_copy2 (GeglBuffer          *src,
                    const GeglRectangle *src_rect,
