@@ -879,8 +879,7 @@ gegl_tile_backend_file_set_property (GObject      *object,
   switch (property_id)
     {
       case PROP_PATH:
-        if (self->path)
-          g_free (self->path);
+        g_free (self->path);
         self->path = g_value_dup_string (value);
         break;
 
@@ -971,11 +970,8 @@ gegl_tile_backend_file_finalize (GObject *object)
       g_free (self->path);
     }
 
-  if (self->monitor)
-    g_object_unref (self->monitor);
-
-  if (self->file)
-    g_object_unref (self->file);
+  g_clear_object (&self->monitor);
+  g_clear_object (&self->file);
 
   g_cond_clear (&self->cond);
 
