@@ -321,8 +321,7 @@ gegl_tile_handler_cache_wash (GeglTileHandlerCache *cache)
     {
       gegl_tile_store (last_dirty);
       gegl_tile_unref (last_dirty);
-      if (last_dirty->tile_storage)
-        g_object_unref (last_dirty->tile_storage);
+      g_clear_object (&last_dirty->tile_storage);
       return TRUE;
     }
   return FALSE;
