@@ -23,10 +23,16 @@
 #include <glib-object.h>
 
 #include "gegl-buffer-types.h"
-#include "gegl-tile-source.h"
-#include "gegl-tile-handler.h"
-#include "gegl-tile-handler-chain.h"
+#include "gegl-tile-handler-cache.h"
+#include "gegl-tile-handler-private.h"
 #include "gegl-tile-storage.h"
+#include "gegl-buffer-private.h"
+
+struct _GeglTileHandlerPrivate
+{
+  GeglTileStorage      *tile_storage;
+  GeglTileHandlerCache *cache;
+};
 
 G_DEFINE_TYPE (GeglTileHandler, gegl_tile_handler, GEGL_TYPE_TILE_SOURCE)
 
@@ -43,13 +49,7 @@ gegl_tile_handler_dispose (GObject *object)
 {
   GeglTileHandler *handler = GEGL_TILE_HANDLER (object);
 
-  if (handler->source != NULL)
-    {
-      if (!(GEGL_IS_TILE_STORAGE (handler->source) &&
-	    gegl_tile_storage_cached_release ((void*)handler->source)))
-        g_object_unref (handler->source);
-      handler->source = NULL;
-    }
+  g_clear_object (&handler->source);
 
   G_OBJECT_CLASS (gegl_tile_handler_parent_class)->dispose (object);
 }
@@ -87,22 +87,6 @@ gegl_tile_handler_get_property (GObject    *gobject,
     }
 }
 
-void
-gegl_tile_handler_set_source (GeglTileHandler *handler,
-                              GeglTileSource  *source)
-{
-  if (handler->source != NULL)
-    g_object_unref (handler->source);
-
-  if (source == NULL)
-    {
-      handler->source = NULL;
-      return;
-    }
-  handler->source = g_object_ref (source);
-
-}
-
 static void
 gegl_tile_handler_set_property (GObject      *gobject,
                                 guint         property_id,
@@ -114,7 +98,7 @@ gegl_tile_handler_set_property (GObject      *gobject,
   switch (property_id)
     {
       case PROP_SOURCE:
-        gegl_tile_handler_set_source (handler, GEGL_TILE_SOURCE (g_value_get_object (value)));
+        gegl_tile_handler_set_source (handler, g_value_get_object (value));
         return;
 
       default:
@@ -126,7 +110,7 @@ gegl_tile_handler_set_property (GObject      *gobject,
 static void
 gegl_tile_handler_class_init (GeglTileHandlerClass *klass)
 {
-  GObjectClass    *gobject_class  = G_OBJECT_CLASS (klass);
+  GObjectClass *gobject_class  = G_OBJECT_CLASS (klass);
 
   gobject_class->set_property = gegl_tile_handler_set_property;
   gobject_class->get_property = gegl_tile_handler_get_property;
@@ -137,14 +121,91 @@ gegl_tile_handler_class_init (GeglTileHandlerClass *klass)
                                                         "GeglBuffer",
                                                         "The tilestore to be a facade for",
                                                         G_TYPE_OBJECT,
-                                                        G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
+                                                        G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT));
+
+  g_type_class_add_private (gobject_class, sizeof (GeglTileHandlerPrivate));
 }
 
 static void
 gegl_tile_handler_init (GeglTileHandler *self)
 {
-  ((GeglTileSource*)self)->command = gegl_tile_handler_command;
-  self->source = NULL;
+  ((GeglTileSource *) self)->command = gegl_tile_handler_command;
+
+  self->priv = G_TYPE_INSTANCE_GET_PRIVATE (self,
+                                            GEGL_TYPE_TILE_HANDLER,
+                                            GeglTileHandlerPrivate);
 }
 
+void
+gegl_tile_handler_set_source (GeglTileHandler *handler,
+                              GeglTileSource  *source)
+{
+  g_set_object (&handler->source, source);
+}
 
+void
+_gegl_tile_handler_set_tile_storage (GeglTileHandler *handler,
+                                     GeglTileStorage *tile_storage)
+{
+  handler->priv->tile_storage = tile_storage;
+}
+
+void
+_gegl_tile_handler_set_cache (GeglTileHandler      *handler,
+                              GeglTileHandlerCache *cache)
+{
+  handler->priv->cache = cache;
+}
+
+GeglTileStorage *
+_gegl_tile_handler_get_tile_storage (GeglTileHandler *handler)
+{
+  return handler->priv->tile_storage;
+}
+
+GeglTileHandlerCache *
+_gegl_tile_handler_get_cache (GeglTileHandler *handler)
+{
+  return handler->priv->cache;
+}
+
+GeglTile *
+gegl_tile_handler_create_tile (GeglTileHandler *handler,
+                               gint             x,
+                               gint             y,
+                               gint             z)
+{
+  GeglTile *tile;
+
+  tile = gegl_tile_new (handler->priv->tile_storage->tile_size);
+
+  tile->tile_storage = handler->priv->tile_storage;
+  tile->x            = x;
+  tile->y            = y;
+  tile->z            = z;
+
+  if (handler->priv->cache)
+    gegl_tile_handler_cache_insert (handler->priv->cache, tile, x, y, z);
+
+  return tile;
+}
+
+GeglTile *
+gegl_tile_handler_dup_tile (GeglTileHandler *handler,
+                            GeglTile        *tile,
+                            gint             x,
+                            gint             y,
+                            gint             z)
+{
+  tile = gegl_tile_dup (tile);
+
+  tile->x = x;
+  tile->y = y;
+  tile->z = z;
+
+  if (handler->priv->cache)
+    gegl_tile_handler_cache_insert (handler->priv->cache, tile, x, y, z);
+
+  return tile;
+}
