@@ -29,9 +29,6 @@
 #define GEGL_HEADER(x) _GEGL_HEADER(x)
 
 #define GEGL_OP_C_FILE GEGL_HEADER(GEGL_OP_C_SOURCE)
-#ifndef GEGL_OP_NO_SOURCE
-#define GEGL_OP_C_FILE_SOURCE GEGL_HEADER(GEGL_OP_C_SOURCE.h)
-#endif
 #endif
 
 #ifndef GEGL_OP_NAME
@@ -297,10 +294,6 @@ gegl_module_register (GTypeModule *module)
 #undef enum_value
 #undef enum_end
 
-#ifdef GEGL_OP_C_FILE_SOURCE
-#include GEGL_OP_C_FILE_SOURCE
-#endif
-
 #ifdef GETTEXT_PACKAGE
 static const gchar *gegl_op_gettext_package G_GNUC_UNUSED = GETTEXT_PACKAGE;
 #else
@@ -496,8 +489,7 @@ set_property (GObject      *gobject,
       break;
 #define property_string(name, label, def_val)                         \
     case PROP_##name:                                                 \
-      if (properties->name)                                           \
-        g_free (properties->name);                                    \
+      g_free (properties->name);                                      \
       properties->name = g_value_dup_string (value);                  \
       break;
 #define property_boolean(name, label, def_val)                        \
@@ -506,38 +498,32 @@ set_property (GObject      *gobject,
       break;
 #define property_file_path(name, label, def_val)                      \
     case PROP_##name:                                                 \
-      if (properties->name)                                           \
-        g_free (properties->name);                                    \
+      g_free (properties->name);                                      \
       properties->name = g_value_dup_string (value);                  \
       break;
 #define property_uri(name, label, def_val)                            \
     case PROP_##name:                                                 \
-      if (properties->name)                                           \
-        g_free (properties->name);                                    \
+      g_free (properties->name);                                      \
       properties->name = g_value_dup_string (value);                  \
       break;
 #define property_object(name, label, def_val)                         \
     case PROP_##name:                                                 \
-      if (properties->name != NULL)                                   \
-         g_object_unref (properties->name);                           \
+      g_clear_object (&properties->name);                             \
       properties->name = g_value_dup_object (value);                  \
       break;
 #define property_curve(name, label, def_val)                          \
     case PROP_##name:                                                 \
-      if (properties->name != NULL)                                   \
-         g_object_unref (properties->name);                           \
+      g_clear_object (&properties->name);                             \
       properties->name = g_value_dup_object (value);                  \
       break;
 #define property_color(name, label, def_val)                          \
     case PROP_##name:                                                 \
-      if (properties->name != NULL)                                   \
-         g_object_unref (properties->name);                           \
+      g_clear_object (&properties->name);                             \
       properties->name = g_value_dup_object (value);                  \
       break;
 #define property_audio_fragment(name, label, def_val)                 \
     case PROP_##name:                                                 \
-      if (properties->name != NULL)                                   \
-         g_object_unref (properties->name);                           \
+      g_clear_object (&properties->name);                             \
       properties->name = g_value_dup_object (value);                  \
       break;
 #define property_path(name, label, def_val)                           \
@@ -612,54 +598,22 @@ static void gegl_op_destroy_notify (gpointer data)
 #define property_double(name, label, def_val)
 #define property_int(name, label, def_val) 
 #define property_string(name, label, ...)      \
-  if (properties->name)                             \
-    {                                               \
-      g_free (properties->name);                    \
-      properties->name = NULL;                      \
-    }
+  g_clear_pointer (&properties->name, g_free);
 #define property_boolean(name, label, def_val)
 #define property_file_path(name, label, def_val)    \
-  if (properties->name)                             \
-    {                                               \
-      g_free (properties->name);                    \
-      properties->name = NULL;                      \
-    }
+  g_clear_pointer (&properties->name, g_free);
 #define property_uri(name, label, def_val)          \
-  if (properties->name)                             \
-    {                                               \
-      g_free (properties->name);                    \
-      properties->name = NULL;                      \
-    }
+  g_clear_pointer (&properties->name, g_free);
 #define property_object(name, label, def_val)       \
-  if (properties->name)                             \
-    {                                               \
-      g_object_unref (properties->name);            \
-      properties->name = NULL;                      \
-    }
+  g_clear_object (&properties->name);
 #define property_curve(name, label, def_val)       \
-  if (properties->name)                             \
-    {                                               \
-      g_object_unref (properties->name);            \
-      properties->name = NULL;                      \
-    }
+  g_clear_object (&properties->name);
 #define property_color(name, label, def_val)       \
-  if (properties->name)                             \
-    {                                               \
-      g_object_unref (properties->name);            \
-      properties->name = NULL;                      \
-    }
+  g_clear_object (&properties->name);
 #define property_audio_fragment(name, label, def_val) \
-  if (properties->name)                             \
-    {                                               \
-      g_object_unref (properties->name);            \
-      properties->name = NULL;                      \
-    }
+  g_clear_object (&properties->name);
 #define property_path(name, label, def_val)         \
-  if (properties->name)                             \
-    {                                               \
-      g_object_unref (properties->name);            \
-      properties->name = NULL;                      \
-    }
+  g_clear_object (&properties->name);
 #define property_pointer(name, label, ...)
 #define property_format(name, label, ...)
 #define property_enum(name, label, ...)
@@ -872,13 +826,6 @@ gegl_op_class_intern_init (gpointer klass)
   gboolean G_GNUC_UNUSED ui_digits_set = FALSE;
   GParamFlags flags G_GNUC_UNUSED = (GParamFlags)(G_PARAM_READWRITE | G_PARAM_CONSTRUCT | GEGL_PARAM_PAD_INPUT);
 
-#ifdef GEGL_OP_C_FILE_SOURCE
-  GeglOperationClass *operation_class = GEGL_OPERATION_CLASS (klass);
-  gegl_operation_class_set_keys (operation_class,
-    "source", op_c_source,
-    NULL);
-#endif
-
   object_class->set_property = set_property;
   object_class->get_property = get_property;
   object_class->constructor  = gegl_op_constructor;
