@@ -35,7 +35,11 @@
 #include "gegl-visitable.h"
 #include "gegl-config.h"
 
+#include "buffer/gegl-region.h"
+
 #include "graph/gegl-visitor.h"
+#include "graph/gegl-callback-visitor.h"
+#include "graph/gegl-node-output-visitable.h"
 
 #include "operation/gegl-operation.h"
 #include "operation/gegl-operations.h"
@@ -98,7 +102,7 @@ static GeglConnection *gegl_node_find_connection          (GeglNode      *sink,
                                                            GeglPad       *sink_pad);
 static void            gegl_node_visitable_iface_init     (gpointer       ginterface,
                                                            gpointer       interface_data);
-static void            gegl_node_visitable_accept         (GeglVisitable *visitable,
+static gboolean        gegl_node_visitable_accept         (GeglVisitable *visitable,
                                                            GeglVisitor   *visitor);
 static GSList*         gegl_node_visitable_depends_on     (GeglVisitable *visitable);
 static void            gegl_node_set_operation_object     (GeglNode      *self,
@@ -219,12 +223,13 @@ gegl_node_init (GeglNode *self)
                                             GEGL_TYPE_NODE,
                                             GeglNodePrivate);
 
-  self->pads        = NULL;
-  self->input_pads  = NULL;
-  self->output_pads = NULL;
-  self->operation   = NULL;
-  self->is_graph    = FALSE;
-  self->cache       = NULL;
+  self->pads             = NULL;
+  self->input_pads       = NULL;
+  self->output_pads      = NULL;
+  self->operation        = NULL;
+  self->is_graph         = FALSE;
+  self->cache            = NULL;
+  self->output_visitable = gegl_node_output_visitable_new (self);
   g_mutex_init (&self->mutex);
 
 }
@@ -252,17 +257,8 @@ gegl_node_dispose (GObject *gobject)
     }
 
   gegl_node_remove_children (self);
-  if (self->cache)
-    {
-      g_object_unref (self->cache);
-      self->cache = NULL;
-    }
-
-  if (self->priv->eval_manager)
-    {
-      g_object_unref (self->priv->eval_manager);
-      self->priv->eval_manager = NULL;
-    }
+  g_clear_object (&self->cache);
+  g_clear_object (&self->priv->eval_manager);
 
   G_OBJECT_CLASS (gegl_node_parent_class)->dispose (gobject);
 }
@@ -275,31 +271,14 @@ gegl_node_finalize (GObject *gobject)
   gegl_node_disconnect_sources (self);
   gegl_node_disconnect_sinks (self);
 
-  if (self->pads)
-    {
-      g_slist_foreach (self->pads, (GFunc) g_object_unref, NULL);
-      g_slist_free (self->pads);
-      self->pads = NULL;
-    }
-
+  g_slist_free_full (self->pads, g_object_unref);
   g_slist_free (self->input_pads);
   g_slist_free (self->output_pads);
 
-  if (self->operation)
-    {
-      g_object_unref (self->operation);
-      self->operation = NULL;
-    }
-
-  if (self->priv->name)
-    {
-      g_free (self->priv->name);
-    }
-
-  if (self->priv->debug_name)
-    {
-      g_free (self->priv->debug_name);
-    }
+  g_clear_object (&self->operation);
+  g_clear_object (&self->output_visitable);
+  g_free (self->priv->name);
+  g_free (self->priv->debug_name);
 
   g_mutex_clear (&self->mutex);
 
@@ -609,36 +588,202 @@ gegl_node_connect_to (GeglNode    *source,
   return gegl_node_connect_from (sink, sink_pad_name, source, source_pad_name);
 }
 
+/* the implementation of gegl_node_invalidated() can use either GeglRegions
+ * or GeglRectangles (bounding boxes) for calculating the invadilated areas
+ * of the nodes in the graph.  The GeglRegion version is more granular,
+ * invalidating exact areas, but has higher overhead, while the GeglRectangle
+ * version in more coarse, potentially over-invalidating (but never under-
+ * invalidating), but has lower overhead.
+ *
+ * TODO: decide if the higher overhead of the GeglRegion version is worth it.
+ */
+/* #define GEGL_NODE_INVALIDATED_USE_REGIONS */
+
+#ifdef GEGL_NODE_INVALIDATED_USE_REGIONS
+
+static gboolean
+gegl_node_invalidated_invalidate_node (GeglNode *node,
+                                       gpointer  data)
+{
+  GHashTable    *regions = data;
+  GeglRegion    *region  = g_hash_table_lookup (regions, node);
+  GeglRectangle *rects;
+  gint           n_rects;
+  GSList        *iter;
+  gint           i;
+
+  node->valid_have_rect = FALSE;
+
+  gegl_region_get_rectangles (region,
+                              &rects, &n_rects);
+
+  for (i = 0; i < n_rects; i++)
+    {
+      if (node->cache)
+        gegl_cache_invalidate (node->cache, &rects[i]);
+
+      g_signal_emit (node, gegl_node_signals[INVALIDATED], 0,
+                     &rects[i], NULL);
+    }
+
+  for (iter = node->priv->sink_connections; iter; iter = g_slist_next (iter))
+    {
+      GeglConnection *connection  = iter->data;
+      GeglNode       *sink_node   = gegl_connection_get_sink_node (connection);
+      GeglPad        *sink_pad    = gegl_connection_get_sink_pad (connection);
+      GeglRegion     *sink_region = g_hash_table_lookup (regions, sink_node);
+
+      if (! sink_region)
+        {
+          sink_region = gegl_region_new ();
+
+          g_hash_table_insert (regions, sink_node, sink_region);
+        }
+
+      if (sink_node->operation)
+        {
+          for (i = 0; i < n_rects; i++)
+            {
+              GeglRectangle invalidated_rect;
+
+              invalidated_rect = gegl_operation_get_invalidated_by_change (
+                sink_node->operation,
+                gegl_pad_get_name (sink_pad), &rects[i]);
+
+              gegl_region_union_with_rect (sink_region, &invalidated_rect);
+            }
+        }
+      else
+        {
+          gegl_region_union (sink_region, region);
+        }
+    }
+
+  g_free (rects);
+
+  return FALSE;
+}
+
 void
 gegl_node_invalidated (GeglNode            *node,
                        const GeglRectangle *rect,
                        gboolean             clear_cache)
 {
+  GHashTable  *regions;
+  GeglVisitor *visitor;
+
   g_return_if_fail (GEGL_IS_NODE (node));
 
   if (!rect)
     rect = &node->have_rect;
 
-  if (node->cache)
-    {
-      if (rect && clear_cache)
-        gegl_buffer_clear (GEGL_BUFFER (node->cache), rect);
+  if (node->cache && clear_cache)
+    gegl_buffer_clear (GEGL_BUFFER (node->cache), rect);
+
+  regions = g_hash_table_new_full (NULL, NULL, NULL,
+                                   (GDestroyNotify) gegl_region_destroy);
+
+  g_hash_table_insert (regions, node, gegl_region_rectangle (rect));
+
+  visitor = gegl_callback_visitor_new (gegl_node_invalidated_invalidate_node,
+                                       regions);
+
+  gegl_visitor_traverse_reverse_topological (visitor,
+                                             gegl_node_get_output_visitable (node));
+
+  g_object_unref (visitor);
+  g_hash_table_unref (regions);
+}
+
+#else /* ! GEGL_NODE_INVALIDATED_USE_REGIONS */
+
+static gboolean
+gegl_node_invalidated_invalidate_node (GeglNode *node,
+                                       gpointer  data)
+{
+  GHashTable          *rects = data;
+  const GeglRectangle *rect  = g_hash_table_lookup (rects, node);
+  GSList              *iter;
 
-      gegl_cache_invalidate (node->cache, rect);
-    }
   node->valid_have_rect = FALSE;
 
+  if (node->cache)
+    gegl_cache_invalidate (node->cache, rect);
+
   g_signal_emit (node, gegl_node_signals[INVALIDATED], 0,
                  rect, NULL);
+
+  for (iter = node->priv->sink_connections; iter; iter = g_slist_next (iter))
+    {
+      GeglConnection *connection = iter->data;
+      GeglNode       *sink_node  = gegl_connection_get_sink_node (connection);
+      GeglPad        *sink_pad   = gegl_connection_get_sink_pad (connection);
+      GeglRectangle  *sink_rect  = g_hash_table_lookup (rects, sink_node);
+
+      if (! sink_rect)
+        {
+          sink_rect = gegl_rectangle_new (0, 0, 0, 0);
+
+          g_hash_table_insert (rects, sink_node, sink_rect);
+        }
+
+      if (sink_node->operation)
+        {
+          GeglRectangle invalidated_rect;
+
+          invalidated_rect = gegl_operation_get_invalidated_by_change (
+            sink_node->operation,
+            gegl_pad_get_name (sink_pad), rect);
+
+          gegl_rectangle_bounding_box (sink_rect, sink_rect, &invalidated_rect);
+        }
+      else
+        {
+          gegl_rectangle_bounding_box (sink_rect, sink_rect, rect);
+        }
+    }
+
+  return FALSE;
 }
 
+void
+gegl_node_invalidated (GeglNode            *node,
+                       const GeglRectangle *rect,
+                       gboolean             clear_cache)
+{
+  GHashTable  *rects;
+  GeglVisitor *visitor;
+
+  g_return_if_fail (GEGL_IS_NODE (node));
+
+  if (!rect)
+    rect = &node->have_rect;
+
+  if (node->cache && clear_cache)
+    gegl_buffer_clear (GEGL_BUFFER (node->cache), rect);
+
+  rects = g_hash_table_new_full (NULL, NULL, NULL, g_free);
+
+  g_hash_table_insert (rects, node, g_memdup (rect, sizeof (GeglRectangle)));
+
+  visitor = gegl_callback_visitor_new (gegl_node_invalidated_invalidate_node,
+                                       rects);
+
+  gegl_visitor_traverse_reverse_topological (visitor,
+                                             gegl_node_get_output_visitable (node));
+
+  g_object_unref (visitor);
+  g_hash_table_unref (rects);
+}
+
+#endif /* ! GEGL_NODE_INVALIDATED_USE_REGIONS */
+
 static void
 gegl_node_source_invalidated (GeglNode            *source,
-                              const GeglRectangle *rect,
-                              gpointer             data)
+                              GeglPad             *destination_pad,
+                              const GeglRectangle *rect)
 {
-  GeglPad       *destination_pad = GEGL_PAD (data);
-  GeglNode      *destination     = gegl_pad_get_node (destination_pad);
+  GeglNode      *destination = gegl_pad_get_node (destination_pad);
   GeglRectangle  dirty_rect;
 
   GEGL_NOTE (GEGL_DEBUG_INVALIDATION, "%s.%s is dirtied from %s (%i,%i %i×%i)",
@@ -662,30 +807,26 @@ gegl_node_source_invalidated (GeglNode            *source,
   gegl_node_invalidated (destination, &dirty_rect, FALSE);
 }
 
-static GSList *
-gegl_node_get_depends_on (GeglNode *self);
+static gboolean
+gegl_node_has_source_node_equals (GeglNode *node,
+                                  gpointer  potential_source)
+{
+  return node == potential_source;
+}
 
 static gboolean
 gegl_node_has_source (GeglNode *self,
                       GeglNode *potential_source)
 {
-  GSList *producers, *p;
-  gboolean found = FALSE;
+  GeglVisitor *visitor;
+  gboolean     found;
 
-  if (self == potential_source)
-    return TRUE;
+  visitor = gegl_callback_visitor_new (gegl_node_has_source_node_equals,
+                                       potential_source);
 
-  producers = gegl_node_get_depends_on (self);
-  for (p = producers; p; p = p->next)
-    {
-      if (p->data == potential_source)
-        found = TRUE;
-      else
-        found = gegl_node_has_source (p->data, potential_source);
-      if (found)
-        break;
-    }
-  g_slist_free (producers);
+  found = gegl_visitor_traverse (visitor, GEGL_VISITABLE (self));
+
+  g_object_unref (visitor);
 
   return found;
 }
@@ -752,10 +893,7 @@ gegl_node_connect_from (GeglNode    *sink,
       real_sink->priv->source_connections = g_slist_prepend (real_sink->priv->source_connections, connection);
       real_source->priv->sink_connections = g_slist_prepend (real_source->priv->sink_connections, connection);
 
-      g_signal_connect (G_OBJECT (real_source), "invalidated",
-                        G_CALLBACK (gegl_node_source_invalidated), sink_pad);
-
-      gegl_node_source_invalidated (real_source, &real_source->have_rect, sink_pad);
+      gegl_node_source_invalidated (real_source, sink_pad, &real_source->have_rect);
 
       return TRUE;
     }
@@ -797,20 +935,7 @@ gegl_node_disconnect (GeglNode    *sink,
       source_pad = gegl_connection_get_source_pad (connection);
       source     = gegl_connection_get_source_node (connection);
 
-      gegl_node_source_invalidated (source, &source->have_rect, sink_pad);
-
-      {
-        /* disconnecting dirt propagation */
-        gulong handler;
-
-        handler = g_signal_handler_find (source, G_SIGNAL_MATCH_DATA,
-                                         gegl_node_signals[INVALIDATED],
-                                         0, NULL, NULL, sink_pad);
-        if (handler)
-          {
-            g_signal_handler_disconnect (source, handler);
-          }
-      }
+      gegl_node_source_invalidated (source, sink_pad, &source->have_rect);
 
       gegl_pad_disconnect (sink_pad, source_pad, connection);
 
@@ -1035,8 +1160,7 @@ gegl_node_blit (GeglNode            *self,
       if (buffer && destination_buf)
         gegl_buffer_get (buffer, roi, scale, format, destination_buf, rowstride, GEGL_ABYSS_NONE);
 
-      if (buffer)
-        g_object_unref (buffer);
+      g_clear_object (&buffer);
     }
   else if (flags & GEGL_BLIT_CACHE)
     {
@@ -1105,11 +1229,11 @@ gegl_node_dump_depends_on (GeglNode *self)
   g_slist_free (depends_on);
 }
 
-static void
+static gboolean
 gegl_node_visitable_accept (GeglVisitable *visitable,
                             GeglVisitor   *visitor)
 {
-  gegl_visitor_visit_node (visitor, (GeglNode *) visitable);
+  return gegl_visitor_visit_node (visitor, (GeglNode *) visitable);
 }
 
 static GSList *
@@ -1257,10 +1381,7 @@ gegl_node_set_operation_object (GeglNode      *self,
   gegl_node_disconnect_sources (self);
   gegl_node_disconnect_sinks (self);
 
-  if (self->operation)
-    g_object_unref (self->operation);
-
-  self->operation = g_object_ref (operation);
+  g_set_object (&self->operation, operation);
 
   /* Delete all the pads from the previous operation */
   while (self->pads)
@@ -1600,8 +1721,7 @@ gegl_node_update_debug_name (GeglNode *node)
 
   g_return_if_fail (GEGL_IS_NODE (node));
 
-  if (node->priv->debug_name)
-    g_free (node->priv->debug_name);
+  g_free (node->priv->debug_name);
 
   if (name && *name)
     new_name = g_strdup_printf ("%s '%s' %p", operation ? operation : "(none)", name, node);
@@ -1741,8 +1861,6 @@ gegl_node_insert_before (GeglNode *self,
   other     = gegl_node_get_producer (self, "input", NULL); /*XXX: handle pad name */
   rectangle = gegl_node_get_bounding_box (to_be_inserted);
 
-  g_signal_handlers_block_matched (other, G_SIGNAL_MATCH_FUNC, 0, 0, 0, gegl_node_source_invalidated, NULL);
-  /* the blocked handler disappears during the relinking */
   gegl_node_link_many (other, to_be_inserted, self, NULL);
 
   /* emit the change ourselves */
@@ -1872,10 +1990,7 @@ gegl_node_get_cache (GeglNode *node)
     }
 
   if (node->cache && gegl_buffer_get_format ((GeglBuffer *)(node->cache)) != format)
-    {
-      g_object_unref (node->cache);
-      node->cache = NULL;
-    }
+    g_clear_object (&node->cache);
 
   if (node->cache)
     return node->cache;
@@ -1905,6 +2020,14 @@ gegl_node_get_cache (GeglNode *node)
   return node->cache;
 }
 
+GeglVisitable *
+gegl_node_get_output_visitable (GeglNode *self)
+{
+  g_return_val_if_fail (GEGL_IS_NODE (self), NULL);
+
+  return self->output_visitable;
+}
+
 const gchar *
 gegl_node_get_name (GeglNode *self)
 {
@@ -1919,9 +2042,7 @@ gegl_node_set_name (GeglNode    *self,
 {
   g_return_if_fail (GEGL_IS_NODE (self));
 
-  if (self->priv->name)
-    g_free (self->priv->name);
-
+  g_free (self->priv->name);
   self->priv->name = g_strdup (name);
 
   gegl_node_update_debug_name (self);
