@@ -36,8 +36,8 @@
 #include "gegl-processor-private.h"
 
 #include "graph/gegl-visitor.h"
+#include "graph/gegl-callback-visitor.h"
 #include "graph/gegl-visitable.h"
-#include "process/gegl-list-visitor.h"
 
 #include "opencl/gegl-cl.h"
 
@@ -160,35 +160,14 @@ gegl_processor_finalize (GObject *self_object)
 {
   GeglProcessor *processor = GEGL_PROCESSOR (self_object);
 
-  if (processor->context)
-    {
-      gegl_operation_context_destroy (processor->context);
-    }
+  g_clear_pointer (&processor->context, (GDestroyNotify) gegl_operation_context_destroy);
 
-  if (processor->node)
-    {
-      g_object_unref (processor->node);
-    }
+  g_clear_object (&processor->node);
+  g_clear_object (&processor->real_node);
+  g_clear_object (&processor->input);
 
-  if (processor->real_node)
-    {
-      g_object_unref (processor->real_node);
-    }
-
-  if (processor->input)
-    {
-      g_object_unref (processor->input);
-    }
-
-  if (processor->queued_region)
-    {
-      gegl_region_destroy (processor->queued_region);
-    }
-
-  if (processor->valid_region)
-    {
-      gegl_region_destroy (processor->valid_region);
-    }
+  g_clear_pointer (&processor->queued_region, (GDestroyNotify) gegl_region_destroy);
+  g_clear_pointer (&processor->valid_region, (GDestroyNotify) gegl_region_destroy);
 
   G_OBJECT_CLASS (gegl_processor_parent_class)->finalize (self_object);
 }
@@ -259,12 +238,8 @@ gegl_processor_set_node (GeglProcessor *processor,
   g_return_if_fail (GEGL_IS_NODE (node));
   g_return_if_fail (node->is_graph || GEGL_IS_OPERATION (node->operation));
 
-  if (processor->node)
-    g_object_unref (processor->node);
-  if (processor->real_node)
-    g_object_unref (processor->real_node);
-
-  processor->node = g_object_ref (node);
+  g_set_object (&processor->node, node);
+  g_clear_object (&processor->real_node);
 
   /* nodes with meta operations are also graphs and can be sinks, so
    * we don't use their output proxy */
@@ -785,6 +760,14 @@ gegl_processor_render (GeglProcessor *processor,
   return !gegl_processor_is_rendered (processor);
 }
 
+static gboolean
+gegl_processor_work_is_opencl_node (GeglNode *node,
+                                    gpointer  data)
+{
+  return GEGL_OPERATION_GET_CLASS (node->operation)->cl_data ||
+         GEGL_OPERATION_GET_CLASS (node->operation)->opencl_support;
+}
+
 /* Will call gegl_processor_render and when there is no more work to be done,
  * it will write the result to the destination */
 gboolean
@@ -798,23 +781,14 @@ gegl_processor_work (GeglProcessor *processor,
       if (gegl_cl_is_accelerated ()
           && processor->chunk_size != GEGL_CL_CHUNK_SIZE)
         {
-          GeglListVisitor *visitor = g_object_new (GEGL_TYPE_LIST_VISITOR, NULL);
-          GList *iterator = NULL;
-          GList *visits_list = NULL;
-          visits_list = gegl_list_visitor_get_dfs_path (visitor, GEGL_VISITABLE (processor->real_node));
+          GeglVisitor *visitor;
 
-          for (iterator = visits_list; iterator; iterator = iterator->next)
-            {
-              GeglNode *node = (GeglNode*) iterator->data;
-              if (GEGL_OPERATION_GET_CLASS(node->operation)->cl_data
-                  || GEGL_OPERATION_GET_CLASS(node->operation)->opencl_support)
-                {
-                  processor->chunk_size = GEGL_CL_CHUNK_SIZE;
-                  break;
-                }
-            }
+          visitor = gegl_callback_visitor_new (gegl_processor_work_is_opencl_node,
+                                               NULL);
+
+          if (gegl_visitor_traverse (visitor, GEGL_VISITABLE (processor->real_node)))
+            processor->chunk_size = GEGL_CL_CHUNK_SIZE;
 
-          g_list_free (visits_list);
           g_object_unref (visitor);
         }
     }
