@@ -353,11 +353,8 @@ operation_process (GeglOperation        *operation,
       success = process (operation, input, aux, output, result, level);
     }
 
-  if (input != NULL)
-    g_object_unref (input);
-
-  if (aux != NULL)
-    g_object_unref (aux);
+  g_clear_object (&input);
+  g_clear_object (&aux);
 
   return success;
 }
@@ -382,7 +379,7 @@ gegl_op_class_init (GeglOpClass *klass)
   gegl_operation_class_set_keys (operation_class,
     "name",        "gegl:watershed-transform",
     "title",       _("Watershed Transform"),
-    "reference-hash", "43ddd80572ab34095298ac7c36368b0c",
+    "reference-hash", "a101c74702f52e4bd2fda04eda56034e",
     "categories",  "hidden",
     "description", _("Labels propagation by watershed transformation"),
     NULL);
