@@ -59,12 +59,12 @@ property_double (highlights_ccorrect, _("Highlights color adjustment"), 50.0)
 static void
 prepare (GeglOperation *operation)
 {
-  const Babl *laba = babl_format ("CIE Lab alpha float");
-  const Babl *lab  = babl_format ("CIE Lab float");
+  const Babl *cie_laba = babl_format ("CIE Lab alpha float");
+  const Babl *cie_lab  = babl_format ("CIE Lab float");
 
-  gegl_operation_set_format (operation, "input",  laba);
-  gegl_operation_set_format (operation, "aux",    lab);
-  gegl_operation_set_format (operation, "output", laba);
+  gegl_operation_set_format (operation, "input",  cie_laba);
+  gegl_operation_set_format (operation, "aux",    cie_lab);
+  gegl_operation_set_format (operation, "output", cie_laba);
 }
 
 static gboolean
@@ -95,7 +95,6 @@ process (GeglOperation       *operation,
 
   gfloat max[4] = { 1.0f, 1.0f, 1.0f, 1.0f };
   gfloat min[4] = { 0.0f, -1.0f, -1.0f, 0.0f };
-  gfloat lmin = 0.0f;
   gfloat lmax = max[0] + fabs(min[0]);
   gfloat halfmax = lmax / 2.0;
   gfloat doublemax = lmax * 2.0;
@@ -137,7 +136,7 @@ process (GeglOperation       *operation,
 
           gfloat la = ta[0];
           gfloat lb = (tb[0] - halfmax) * SIGN(-highlights) * SIGN(lmax - la) + halfmax;
-          lb = CLAMP(lb, lmin, lmax);
+
           lref = copysignf(fabs(la) > low_approximation ? 1.0f / fabs(la) : 1.0f / low_approximation, la);
           href = copysignf(
               fabs(1.0f - la) > low_approximation ? 1.0f / fabs(1.0f - la) : 1.0f / low_approximation, 1.0f - la);
@@ -203,8 +202,6 @@ process (GeglOperation       *operation,
   return TRUE;
 }
 
-/*
-
 #include "opencl/gegl-cl.h"
 
 #include "opencl/shadows-highlights-correction.cl.h"
@@ -273,8 +270,6 @@ error:
   return TRUE;
 }
 
-*/
-
 static GeglRectangle
 get_bounding_box (GeglOperation *self)
 {
@@ -298,14 +293,10 @@ gegl_op_class_init (GeglOpClass *klass)
 
   operation_class->prepare          = prepare;
   operation_class->get_bounding_box = get_bounding_box;
-  
+  operation_class->opencl_support   = TRUE;
 
   point_composer_class->process     = process;
-
-  /*
   point_composer_class->cl_process  = cl_process;
-  operation_class->opencl_support   = TRUE;
-   */
 
   gegl_operation_class_set_keys (operation_class,
     "name",        "gegl:shadows-highlights-correction",
